'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var immer = require('immer');
var bind = _interopDefault(require('auto-bind'));
var isEqual = _interopDefault(require('lodash/isEqual'));
var isFunction = _interopDefault(require('lodash/isFunction'));

var classCallCheck = function (instance, Constructor) {
  if (!(instance instanceof Constructor)) {
    throw new TypeError("Cannot call a class as a function");
  }
};

var createClass = function () {
  function defineProperties(target, props) {
    for (var i = 0; i < props.length; i++) {
      var descriptor = props[i];
      descriptor.enumerable = descriptor.enumerable || false;
      descriptor.configurable = true;
      if ("value" in descriptor) descriptor.writable = true;
      Object.defineProperty(target, descriptor.key, descriptor);
    }
  }

  return function (Constructor, protoProps, staticProps) {
    if (protoProps) defineProperties(Constructor.prototype, protoProps);
    if (staticProps) defineProperties(Constructor, staticProps);
    return Constructor;
  };
}();

var defineProperty = function (obj, key, value) {
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }

  return obj;
};

var Store = function () {
  createClass(Store, [{
    key: '__isStore',
    get: function get$$1() {
      return true;
    }
  }, {
    key: 'state',
    get: function get$$1() {
      return this._state;
    }
  }]);

  function Store(state) {
    classCallCheck(this, Store);
    this._state = {};
    this._subscribers = [];

    this._state = immer.produce(state, function () {});
    bind(this);
  }

  createClass(Store, [{
    key: 'getState',
    value: function getState() {
      return this.state;
    }
  }, {
    key: 'setState',
    value: function setState(mutation) {
      var _this = this;

      if (Array.isArray(mutation)) {
        mutation.map(this.setState);
      } else {
        for (var _len = arguments.length, args = Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
          args[_key - 1] = arguments[_key];
        }

        this._state = immer.produce.apply(undefined, [this.state, isFunction(mutation) ? mutation : function () {
          return Object.assign({}, _this.state, mutation);
        }].concat(args));
      }
      this._notify();
      return this.state;
    }

    /**
     * subscribe - Add subscriber
     *
     * @param {Function} handler subscriber callback
     * @param {Object} option Options for subscription
     * @param {boolean} option.immediate Whether to run sub
     * @param {Function} option.getter getters for data subscription
     *
     * @returns {Function} unsubscriber
     */

  }, {
    key: 'subscribe',
    value: function subscribe(handler) {
      var _this2 = this;

      var _ref = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {},
          _ref$immediate = _ref.immediate,
          immediate = _ref$immediate === undefined ? false : _ref$immediate,
          _ref$getter = _ref.getter,
          getter = _ref$getter === undefined ? function (state) {
        return state;
      } : _ref$getter;

      // TODO: check if the getter is not cachable here
      this._subscribers.push({ handler: handler, getter: getter });
      if (immediate) {
        this._notify();
      }
      return function () {
        return _this2.unsubscribe(handler);
      };
    }

    /**
     * unsubscribe - Remove subscriber
     *
     * @param {Function} handler subscriber to remvoe
     *
     */

  }, {
    key: 'unsubscribe',
    value: function unsubscribe(handler) {
      this._subscribers.splice(this._subscribers.findIndex(function (sub) {
        return sub.handler === handler;
      }), 1);
    }
  }, {
    key: '_notify',
    value: function _notify() {
      var _this3 = this;

      this._subscribers.forEach(function (sub) {
        // Notify if getterCache is updated
        var getterCache = sub.getter(_this3.state);
        if (!isEqual(getterCache, sub.getterCache)) {
          sub.getterCache = getterCache;
          sub.handler(getterCache);
        }
      });
    }
  }]);
  return Store;
}();

var register = function register(state) {
  var _ref2 = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {},
      name = _ref2.name,
      _ref2$plugins = _ref2.plugins,
      plugins = _ref2$plugins === undefined ? [] : _ref2$plugins;

  var store = new Store(state);
  store.name = name;

  return plugins.reduce(function (store, plugin) {
    return plugin.apply(store) || store;
  }, store);
};

var store = register;

var observableSymbol = function observableSymbol() {
  return typeof Symbol === 'function' && Symbol.observable || '@@observable';
};

// Returns an observable stream
var toStream = function toStream(store) {
  var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

  return defineProperty({
    subscribe: function subscribe() {
      var observer = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : function () {};
      return {
        unsubscribe: store.subscribe(typeof observer === 'function' ? function (selected) {
          return observer(selected);
        } : function (selected) {
          return observer.next(selected);
        }, Object.assign({}, options, { immediate: true }))
      };
    }
  }, observableSymbol(), function () {
    return this;
  });
};

exports.register = register;
exports.store = store;
exports.toStream = toStream;
//# sourceMappingURL=reim.cjs.js.map
